#!/usr/bin/env python

import sys

import json

from charmhelpers.contrib.openstack.utils import configure_installation_source

from charmhelpers.core.hookenv import (
    Hooks,
    UnregisteredHookError,
    config,
    local_unit,
    log,
    relation_get
)

from charmhelpers.core.host import (
    restart_on_change,
    service_restart
)

from charmhelpers.fetch import (
    apt_install,
    apt_upgrade,
    configure_sources
)

from contrail_control_utils import (
    fix_permissions,
    write_control_config
)

PACKAGES = [ "contrail-control" ]

hooks = Hooks()

@hooks.hook("config-changed")
def config_changed():
    pass

@hooks.hook("contrail-ifmap-relation-changed")
def contrail_ifmap_changed():
    creds = relation_get("creds")
    creds = json.loads(creds) if creds else {}
    if local_unit() not in creds:
        log("Relation not ready")
        return
    contrail_ifmap_relation()

@hooks.hook("contrail-ifmap-relation-departed")
@hooks.hook("contrail-ifmap-relation-broken")
@restart_on_change({"/etc/contrail/contrail-control.conf": ["contrail-control"],
                    "/etc/contrail/control-node.conf": ["contrail-control"]})
def contrail_ifmap_relation():
    write_control_config()

@hooks.hook()
def install():
    configure_installation_source(config("openstack-origin"))
    configure_sources(True, "install-sources")
    apt_upgrade(fatal=True, dist=True)
    apt_install(PACKAGES, fatal=True)
    fix_permissions()

def main():
    try:
        hooks.execute(sys.argv)
    except UnregisteredHookError as e:
        log("Unknown hook {} - skipping.".format(e))

@hooks.hook("upgrade-charm")
def upgrade_charm():
    write_control_config()
    service_restart("contrail-control")

if __name__ == "__main__":
    main()
