import os
from socket import gethostbyname
from subprocess import CalledProcessError, check_output

import apt_pkg
from apt_pkg import version_compare
import json

from charmhelpers.core.hookenv import (
    local_unit,
    related_units,
    relation_get,
    relation_ids
)

from charmhelpers.core.templating import render

apt_pkg.init()

def dpkg_version(pkg):
    try:
        return check_output(["dpkg-query", "-f", "${Version}\\n", "-W", pkg]).rstrip()
    except CalledProcessError:
        return None

CONTRAIL_VERSION = dpkg_version("contrail-control")

def contrail_ifmap_ctx():
    ctxs = []
    unit = local_unit()
    for rid in relation_ids("contrail-ifmap"):
        for u in related_units(rid):
            ctx = {}
            ctx["discovery_server"] = gethostbyname(relation_get("private-address", u, rid))
            creds = relation_get("creds", u, rid)
            if creds:
                creds = json.loads(creds)
                if unit in creds:
                    cs = creds[unit]
                    ctx["ifmap_user"] = cs["username"]
                    ctx["ifmap_password"] = cs["password"]
            ctxs.append(ctx)
    return ctxs[0] if ctxs else {}

def fix_permissions():
    os.chmod("/etc/contrail", 0755)
    os.chown("/etc/contrail", 0, 0)

def write_control_config():
    ctx = contrail_ifmap_ctx()
    target = "/etc/contrail/contrail-control.conf" \
             if version_compare(CONTRAIL_VERSION, "2.0") >= 0 \
             else "/etc/contrail/control-node.conf"
    render("control-node.conf", target, ctx, "root", "contrail", 0440)
